@@ -12,7 +12,7 @@ Perl::MinimumVersion - Find a minimum required version of perl for Perl code
   $object = Perl::MinimumVersion->new( $filename );
   $object = Perl::MinimumVersion->new( \$source  );
   $object = Perl::MinimumVersion->new( $ppi_document );
-  
+
   # Find the minimum version
   $version = $object->minimum_version;
 
@@ -34,59 +34,107 @@ covers it.
 
 =cut
 
-use 5.005;
+use 5.006;
 use strict;
 use version      ();
 use Carp         ();
+use Exporter     ();
 use List::Util   ();
-use Params::Util '_INSTANCE', '_CLASS';
-use PPI::Util    '_Document';
+use Params::Util ('_INSTANCE', '_CLASS');
+use PPI::Util    ('_Document');
 use PPI          ();
-use base 'Exporter';
+use Perl::Critic::Utils 1.104 qw{
+	:classification
+	:ppi
+};
 
-use vars qw{$VERSION @EXPORT_OK %CHECKS %MATCHES};
+use Perl::MinimumVersion::Reason ();
+
+use vars qw{$VERSION @ISA @EXPORT_OK %CHECKS %MATCHES};
 BEGIN {
-	$VERSION = '0.15';
+	$VERSION = '1.23_01';
+
+	# Only needed for dev releases, comment out otherwise
+	$VERSION = eval $VERSION;
 
 	# Export the PMV convenience constant
+	@ISA       = 'Exporter';
 	@EXPORT_OK = 'PMV';
 
 	# The primary list of version checks
 	%CHECKS = (
+		_perl_5010_pragmas      => version->new('5.010'),
+		_perl_5010_operators    => version->new('5.010'),
+		_perl_5010_magic        => version->new('5.010'),
+
 		# Various small things
-		_bugfix_magic_errno   => version->new('5.008.003'),
-		_unquoted_versions    => version->new('5.008.001'),
-		_constant_hash        => version->new('5.008'),
-		_use_base_exporter    => version->new('5.008'),
+		_bugfix_magic_errno     => version->new('5.008.003'),
+		_unquoted_versions      => version->new('5.008.001'),
+		_perl_5008_pragmas      => version->new('5.008'),
+		_constant_hash          => version->new('5.008'),
+		_use_base_exporter      => version->new('5.008'),
+		_local_soft_reference   => version->new('5.008'),
+		_use_carp_version       => version->new('5.008'),
 
 		# Included in 5.6. Broken until 5.8
-		_pragma_utf8          => version->new('5.008'),
+		_pragma_utf8            => version->new('5.008'),
 
-		_perl_5006_pragmas    => version->new('5.006'),
-		_any_our_variables    => version->new('5.006'),
-		_any_binary_literals  => version->new('5.006'),
-		_magic_version        => version->new('5.006'),
-		_any_attributes       => version->new('5.006'),
+		_perl_5006_pragmas      => version->new('5.006'),
+		_any_our_variables      => version->new('5.006'),
+		_any_binary_literals    => version->new('5.006'),
+		_any_version_literals   => version->new('5.006'), #v-string
+		_magic_version          => version->new('5.006'),
+		_any_attributes         => version->new('5.006'),
+		_any_CHECK_blocks       => version->new('5.006'),
+		_three_argument_open    => version->new('5.006'),
+		_weaken                 => version->new('5.006'),
+		_mkdir_1_arg            => version->new('5.006'),
 
-		_perl_5005_pragmas    => version->new('5.005'),
-		_perl_5005_modules    => version->new('5.005'),
-		_any_tied_arrays      => version->new('5.005'),
-		_any_quotelike_regexp => version->new('5.005'),
-		_any_INIT_blocks      => version->new('5.005'),
-		);
+		_any_qr_tokens          => version->new('5.005.03'),
+		_perl_5005_pragmas      => version->new('5.005'),
+		_perl_5005_modules      => version->new('5.005'),
+		_any_tied_arrays        => version->new('5.005'),
+		_any_quotelike_regexp   => version->new('5.005'),
+		_any_INIT_blocks        => version->new('5.005'),
+		_substr_4_arg           => version->new('5.005'),
+		_splice_negative_length => version->new('5.005'),
+
+		_postfix_foreach        => version->new('5.004.05'),
+	);
 
 	# Predefine some indexes needed by various check methods
 	%MATCHES = (
+		_perl_5010_pragmas => {
+			mro     => 1,
+			feature => 1,
+		},
+		_perl_5010_operators => {
+			'//'  => 1,
+			'//=' => 1,
+			'~~'  => 1,
+		},
+		_perl_5010_magic => {
+			'%+' => 1,
+			'%-' => 1,
+		},
+		_perl_5008_pragmas => {
+			threads           => 1,
+			'threads::shared' => 1,
+			sort              => 1,
+		},
 		_perl_5006_pragmas => {
-			warnings   => 1,
-			attributes => 1,
-			open       => 1,
-			filetest   => 1,
+			warnings             => 1, #may be ported into older version
+			'warnings::register' => 1,
+			attributes           => 1,
+			open                 => 1,
+			filetest             => 1,
+			charnames            => 1,
+			bytes                => 1,
 		},
 		_perl_5005_pragmas => {
-			re         => 1,
-			fields     => 1,
-			attr       => 1,
+			re     => 1,
+			fields => 1, # can be installed from CPAN, with base.pm
+			attr   => 1,
 		},
 	);
 }
@@ -134,8 +182,7 @@ sub new {
 		explicit => undef,
 		syntax   => undef,
 		external => undef,
-
-		}, $class;
+	}, $class;
 
 	$self;
 }
@@ -149,7 +196,9 @@ back out of the version checker.
 
 =cut
 
-sub Document { $_[0]->{Document} }
+sub Document {
+	$_[0]->{Document}
+}
 
 
 
@@ -173,7 +222,7 @@ Returns a L<version> object, or C<undef> on error.
 =cut
 
 sub minimum_version {
-	my $self    = _self(@_) or return undef;
+	my $self    = _SELF(\@_) or return undef;
 	my $minimum = $self->{default}; # Sensible default
 
 	# Is the explicit version greater?
@@ -186,16 +235,10 @@ sub minimum_version {
 	# Is the syntax version greater?
 	# Since this is the most expensive operation (for this file),
 	# we need to be careful we don't run things we don't need to.
-	if ( defined $self->{syntax} ) {
-		if ( $self->{syntax} and $self->{syntax} > $minimum ) {
-			$minimum = $self->{syntax};
-		}
-	} else {
-		my $syntax = $self->minimum_syntax_version;
-		return undef unless defined $syntax;
-		if ( $syntax and $syntax > $minimum ) {
-			$minimum = $syntax;
-		}
+	my $syntax = $self->minimum_syntax_version;
+	return undef unless defined $syntax;
+	if ( $syntax and $syntax > $minimum ) {
+		$minimum = $syntax;
 	}
 
 	### FIXME - Disabled until minimum_external_version completed
@@ -209,6 +252,27 @@ sub minimum_version {
 	$minimum;
 }
 
+sub minimum_reason {
+	my $self    = _SELF(\@_) or return undef;
+	my $minimum = $self->default_reason; # Sensible default
+
+	# Is the explicit version greater?
+	my $explicit = $self->minimum_explicit_version;
+	return undef unless defined $explicit;
+	if ( $explicit and $explicit > $minimum ) {
+		$minimum = $explicit;
+	}
+
+}
+
+sub default_reason {
+	Perl::MinimumVersion->new(
+		rule    => 'default',
+		version => $_[0]->{default},
+		element => undef,
+	);
+}
+
 =pod
 
 =head2 minimum_explicit_version
@@ -228,11 +292,17 @@ or C<undef> on error.
 =cut
 
 sub minimum_explicit_version {
-	my $self = _self(@_) or return undef;
+	my $self   = _SELF(\@_) or return undef;
+	my $reason = $self->minimum_explicit_reason(@_);
+	return $reason ? $reason->version : $reason;
+}
+
+sub minimum_explicit_reason {
+	my $self = _SELF(\@_) or return undef;
 	unless ( defined $self->{explicit} ) {
 		$self->{explicit} = $self->_minimum_explicit_version;
 	}
-	$self->{explicit};
+	return $self->{explicit};
 }
 
 sub _minimum_explicit_version {
@@ -241,13 +311,25 @@ sub _minimum_explicit_version {
 		$_[1]->isa('PPI::Statement::Include') or return '';
 		$_[1]->version                        or return '';
 		1;
-		} );
+	} );
 	return $explicit unless $explicit;
 
-	# Convert to version objects
-	List::Util::max map { version->new($_) }
-	                map { $_->version      }
-	                @$explicit;
+	# Find the highest version
+	my $max     = undef;
+	my $element = undef;
+	foreach my $include ( @$explicit ) {
+		my $version = version->new($include->version);
+		if ( $version > $max or not $element ) {
+			$max     = $version;
+			$element = $include;
+		}
+	}
+
+	return Perl::MinimumVersion::Reason->new(
+		rule    => 'explicit',
+		version => $max,
+		element => $element,
+	);
 }
 
 =pod
@@ -270,7 +352,7 @@ artifically low results, but should not artificially high results.
 
 For example, if C<minimum_syntax_version> returned 5.006, you can be
 confident it will not run on anything lower, although there is a chance
-that during actual execution it may use some untestable  feature that
+that during actual execution it may use some untestable feature that
 creates a dependency on a higher version.
 
 Returns a L<version> object, false if no dependencies could be found,
@@ -279,9 +361,15 @@ or C<undef> on error.
 =cut
 
 sub minimum_syntax_version {
-	my $self  = _self(@_) or return undef;
-	my $limit = ref($_[0]) ? $_[1] : $_[2];
-	if ( defined $limit and ! _INSTANCE($limit, 'version') ) {
+	my $self   = _SELF(\@_) or return undef;
+	my $reason = $self->minimum_syntax_reason(@_);
+	return $reason ? $reason->version : $reason;
+}
+
+sub minimum_syntax_reason {
+	my $self  = _SELF(\@_) or return undef;
+	my $limit = shift;
+	if ( defined $limit and not _INSTANCE($limit, 'version') ) {
 		$limit = version->new("$limit");
 	}
 	if ( defined $self->{syntax} ) {
@@ -301,8 +389,12 @@ sub minimum_syntax_version {
 	# If we found a value, it will be stable, cache it.
 	# If we did NOT, don't cache as subsequent runs without
 	# the filter may find a version.
-	$self->{syntax} = $syntax if $syntax;
-	return $syntax;
+	if ( $syntax ) {
+		$self->{syntax} = $syntax;
+		return $self->{syntax};
+	}
+
+	return '';
 }
 
 sub _minimum_syntax_version {
@@ -312,12 +404,25 @@ sub _minimum_syntax_version {
 	# Always check in descending version order.
 	# By doing it this way, the version of the first check that matches
 	# is also the version of the document as a whole.
-	my $check = List::Util::first { $self->$_()    }
-	            sort { $CHECKS{$b} <=> $CHECKS{$a} }
-	            grep { $CHECKS{$_}  >  $filter     }
-	            keys %CHECKS;
+	my @rules = sort {
+		$CHECKS{$b} <=> $CHECKS{$a}
+	} grep {
+		$CHECKS{$_} > $filter
+	} keys %CHECKS;
 
-	$check ? $CHECKS{$check} : '';
+	foreach my $rule ( @rules ) {
+		my $result = $self->$rule() or next;
+
+		# Create the result object
+		return Perl::MinimumVersion::Reason->new(
+			rule    => $rule,
+			version => $CHECKS{$rule},
+			element => _INSTANCE($result, 'PPI::Element'),
+		);
+	}
+
+	# Found nothing of interest
+	return '';
 }
 
 =pod
@@ -337,7 +442,13 @@ C<undef> on error.
 =cut
 
 sub minimum_external_version {
-	my $self = _self(@_) or return undef;
+	my $self   = _SELF(\@_) or return undef;
+	my $reason = $self->minimum_explicit_reason(@_);
+	return $reason ? $reason->version : $reason;
+}
+
+sub minimum_external_reason {
+	my $self = _SELF(\@_) or return undef;
 	unless ( defined $self->{external} ) {
 		$self->{external} = $self->_minimum_external_version;
 	}
@@ -348,6 +459,47 @@ sub _minimum_external_version {
 	Carp::croak("Perl::MinimumVersion::minimum_external_version is not implemented");
 }
 
+=pod
+
+=head2 version_markers
+
+This method returns a list of pairs in the form:
+
+  ($version, \@markers)
+
+Each pair represents all the markers that could be found indicating that the
+version was the minimum needed version.  C<@markers> is an array of strings.
+Currently, these strings are not as clear as they might be, but this may be
+changed in the future.  In other words: don't rely on them as specific
+identifiers.
+
+=cut
+
+sub version_markers {
+	my $self = _SELF(\@_) or return undef;
+
+	my %markers;
+
+	if ( my $explicit = $self->minimum_explicit_version ) {
+		$markers{ $explicit } = [ 'explicit' ];
+	}
+
+	foreach my $check ( keys %CHECKS ) {
+		next unless $self->$check();
+		my $markers = $markers{ $CHECKS{$check} } ||= [];
+		push @$markers, $check;
+	}
+
+	my @rv;
+	my %marker_ver = map { $_ => version->new($_) } keys %markers;
+
+	foreach my $ver ( sort { $marker_ver{$b} <=> $marker_ver{$a} } keys %markers ) {
+		push @rv, $marker_ver{$ver} => $markers{$ver};
+	}
+
+	return @rv;
+}
+
 
 
 
@@ -355,6 +507,39 @@ sub _minimum_external_version {
 #####################################################################
 # Version Check Methods
 
+sub _perl_5010_pragmas {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Include')
+		and
+		$MATCHES{_perl_5010_pragmas}->{$_[1]->pragma}
+	} );
+}
+
+sub _perl_5010_operators {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Magic')
+		and
+		$MATCHES{_perl_5010_operators}->{$_[1]->content}
+	} );
+}
+
+sub _perl_5010_magic {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Operator')
+		and
+		$MATCHES{_perl_5010_magic}->{$_[1]->content}
+	} );
+}
+
+sub _perl_5008_pragmas {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Include')
+		and
+		$MATCHES{_perl_5008_pragmas}->{$_[1]->pragma}
+	} );
+}
+
+# FIXME: Needs to be upgraded to return something
 sub _bugfix_magic_errno {
 	my $Document = shift->Document;
 	$Document->find_any( sub {
@@ -372,7 +557,7 @@ sub _bugfix_magic_errno {
 
 # version->new(5.005.004);
 sub _unquoted_versions {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Token::Number')       or return '';
 		$_[1]->{_subtype}                      or return '';
 		$_[1]->{_subtype} eq 'base256'         or return '';
@@ -394,7 +579,7 @@ sub _unquoted_versions {
 }
 
 sub _pragma_utf8 {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		(
@@ -408,7 +593,7 @@ sub _pragma_utf8 {
 
 # Check for the use of 'use constant { ... }'
 sub _constant_hash {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		$_[1]->type
@@ -422,7 +607,7 @@ sub _constant_hash {
 }
 
 sub _perl_5006_pragmas {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		$MATCHES{_perl_5006_pragmas}->{$_[1]->pragma}
@@ -430,7 +615,7 @@ sub _perl_5006_pragmas {
 }
 
 sub _any_our_variables {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Variable')
 		and
 		$_[1]->type eq 'our'
@@ -438,7 +623,7 @@ sub _any_our_variables {
 }
 
 sub _any_binary_literals {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Token::Number')
 		and
 		$_[1]->{_subtype}
@@ -447,8 +632,15 @@ sub _any_binary_literals {
 	} );	
 }
 
+sub _any_version_literals {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Number::Version')
+	} );	
+}
+
+
 sub _magic_version {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Token::Magic')
 		and
 		$_[1]->content eq '$^V'
@@ -456,11 +648,23 @@ sub _magic_version {
 }
 
 sub _any_attributes {
-	shift->Document->find_any( 'Token::Attribute' );
+	shift->Document->find_first( 'Token::Attribute' );
+}
+
+sub _any_CHECK_blocks {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Scheduled')
+		and
+		$_[1]->type eq 'CHECK'
+	} );
+}
+
+sub _any_qr_tokens {
+	shift->Document->find_first( 'Token::QuoteLike::Regexp' );
 }
 
 sub _perl_5005_pragmas {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		$MATCHES{_perl_5005_pragmas}->{$_[1]->pragma}
@@ -470,26 +674,29 @@ sub _perl_5005_pragmas {
 # A number of modules are highly indicative of using techniques
 # that are themselves version-dependant.
 sub _perl_5005_modules {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		$_[1]->module
 		and (
 			$_[1]->module eq 'Tie::Array'
 			or
-			$_[1]->module =~ /\bException\b/
+			($_[1]->module =~ /\bException\b/ and
+				$_[1]->module !~ /^(?:CPAN)::/)
 			or
 			$_[1]->module =~ /\bThread\b/
 			or
 			$_[1]->module =~ /^Error\b/
 			or
 			$_[1]->module eq 'base'
+			or
+			$_[1]->module eq 'Errno'
 		)
 	} );
 }
 
 sub _any_tied_arrays {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Sub')
 		and
 		$_[1]->name eq 'TIEARRAY'
@@ -497,11 +704,11 @@ sub _any_tied_arrays {
 }
 
 sub _any_quotelike_regexp {
-	shift->Document->find_any( 'Token::QuoteLike::Regexp' );
+	shift->Document->find_first( 'Token::QuoteLike::Regexp' );
 }
 
 sub _any_INIT_blocks {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Scheduled')
 		and
 		$_[1]->type eq 'INIT'
@@ -510,7 +717,7 @@ sub _any_INIT_blocks {
 
 # use base 'Exporter' doesn't work reliably everywhere until 5.008
 sub _use_base_exporter {
-	shift->Document->find_any( sub {
+	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
 		and
 		$_[1]->module eq 'base'
@@ -521,6 +728,171 @@ sub _use_base_exporter {
 	} );
 }
 
+# You can't localize a soft reference
+sub _local_soft_reference {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Variable')  or return '';
+		$_[1]->type eq 'local'                  or return '';
+
+		# The second child should be a '$' cast.
+		my @child = $_[1]->schildren;
+		scalar(@child) >= 2                     or return '';
+		$child[1]->isa('PPI::Token::Cast')      or return '';
+		$child[1]->content eq '$'               or return '';
+
+		# The third child should be a block
+		$child[2]->isa('PPI::Structure::Block') or return '';
+
+		# Inside the block should be a string in a statement
+		my $statement = $child[2]->schild(0)    or return '';
+		$statement->isa('PPI::Statement')       or return '';
+		my $inside = $statement->schild(0)      or return '';
+		$inside->isa('PPI::Token::Quote')       or return '';
+
+		# This is indeed a localized soft reference
+		return 1;
+	} );
+}
+
+# Carp.pm did not have a $VERSION in 5.6.2
+# Therefore, even "use Carp 0" imposes a 5.8.0 dependency.
+sub _use_carp_version {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Include') or return '';
+		$_[1]->module eq 'Carp'               or return '';
+
+		my $version = $_[1]->module_version;
+		return !! ( defined $version and length "$version" );
+	} );
+}
+
+sub _three_argument_open {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement')  or return '';
+		my @children=$_[1]->children;
+		#@children >= 7                or return '';
+		my $main_element=$children[0];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'open'       or return '';
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments > 2 ) {
+			return 1;
+		}
+		return '';
+	} );
+}
+
+
+sub _substr_4_arg {
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'substr'       or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments > 3 ) {
+			return 1;
+		}
+		return '';
+	} );
+}
+
+sub _mkdir_1_arg {
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'mkdir'       or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments != 2 ) {
+			return 1;
+		}
+		return '';
+	} );
+}
+
+sub _splice_negative_length {
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'splice'       or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments < 3 ) {
+			return '';
+		}
+		my $arg=$arguments[2];
+		if (ref($arg) eq 'ARRAY') {
+		  $arg=$arg->[0];
+		}
+		if ($arg->isa('PPI::Token::Number')) {
+			if ($arg->literal<0) {
+				return 1;
+			} else {
+				return '';
+			}
+		}
+		return '';
+	} );
+
+}
+
+sub _postfix_foreach {
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'foreach'       or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+		my $stmnt = $main_element->statement();
+		return '' if !$stmnt;
+		return '' if $stmnt->isa('PPI::Statement::Compound');
+		return 1;
+	} );
+}
+
+# weak references require perl 5.6
+# will not work in case of importing several
+sub _weaken {
+	shift->Document->find_first( sub {
+		(
+			$_[1]->isa('PPI::Statement::Include')
+			and
+			$_[1]->module eq 'Scalar::Util'
+			and
+			$_[1]->content =~ /[^:]\b(?:weaken|isweak)\b[^:]/
+		)
+		or
+		(
+			$_[1]->isa('PPI::Token::Word')
+			and
+			(
+				$_[1]->content eq 'Scalar::Util::isweak'
+				or
+				$_[1]->content eq 'Scalar::Util::weaken'
+			)
+			#and
+			#is_function_call($_[1])
+		)
+	} );
+}
+
 
 
 
@@ -529,14 +901,21 @@ sub _use_base_exporter {
 # Support Functions
 
 # Let sub be a function, object method, and static method
-sub _self {
-	if ( _INSTANCE($_[0], 'Perl::MinimumVersion') ) {
-		return shift;
+sub _SELF {
+	my $param = shift;
+	if ( _INSTANCE($param->[0], 'Perl::MinimumVersion') ) {
+		return shift @$param;
 	}
-	if ( _CLASS($_[0]) and $_[0]->isa('Perl::MinimumVersion') ) {
-		return shift->new($_[0]);
+	if (
+		_CLASS($param->[0])
+		and
+		$param->[0]->isa('Perl::MinimumVersion')
+	) {
+		my $class   = shift @$param;
+		my $options = shift @$param;
+		return $class->new($options);
 	}
-	Perl::MinimumVersion->new($_[0]);
+	Perl::MinimumVersion->new(shift @$param);
 }
 
 # Find the maximum version, ignoring problems
@@ -553,8 +932,8 @@ sub _max {
 
 =head1 BUGS
 
-It's very early days, so this probably doesn't catch anywhere near enough
-syntax cases, and I personally don't know enough of them.
+B<Perl::MinimumVersion> does a reasonable job of catching the best-known
+explicit version dependencies.
 
 B<However> it is exceedingly easy to add a new syntax check, so if you
 find something this is missing, copy and paste one of the existing
@@ -565,11 +944,15 @@ I don't even need an entire diff... just the function and version.
 
 =head1 TO DO
 
-- Write lots more version checkers
+B<Write lots more version checkers>
 
-- Write the explicit version checker
+- Perl 5.10 operators and language structures
 
-- Write the recursive module descend stuff
+- Three-argument open
+
+B<Write the explicit version checker>
+
+B<Write the recursive module descend stuff>
 
 =head1 SUPPORT
 
@@ -589,7 +972,7 @@ L<http://ali.as/>, L<PPI>, L<version>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005, 2006 Adam Kennedy.
+Copyright 2005 - 2010 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
